/********************************************************************
 FileName:      DC-DC_PWM_16F1829_main.c

********************************************************************
 File Description:

 Change History:
  Rev   Description
  ----  -----------------------------------------
  1.0   Initial release


********************************************************************/

#define PSVAL       16               //Prescaler
#define TM0VAL      19              // reload value for 50 uSec > 1 mSec

#define	OFFSET              1
#define USE_OR_MASKS

#define	ADCON0SET	0b00001101	//<6:2>CHSEL: 00011=AN3, <1>GO/DONE, <0>ADON

#define	ADCON1SET	0b11010011	//<7>ADFM 1=right, <6:4>ADCS 101=Fosc/16, <3:2>N/A, <1:0>ADPREF 11=FVR

#define	ANSELASET	0b00000000	//<0,1,2,x,4>=ANSA0, ANSA1, ANSA2, ANSA4
					//RA4=AN3 (VBAT), RA2=AN2, RA1=AN1, RA0=AN0

#define	ANSELBSET	0b00000000	//<7:4> ANSELB7:4

#define	ANSELCSET	0b00000000	//

#define APFCON0SET   0b10000000  //<7>RXDTSEL 1=RA1, <6>SDO1SEL RC7, <5>SS1SEL RC6, <3>T1GSEL 0=RA4 <2>TXCKSEL 0=RB7

#define APFCON1SET   0b10000000  //<7> 

#define	CCP1CONSET	0b00000000	//<7:6>P1M, <5:4>DC1B, <3:0>CCP1M

#define	FVRCONSET	0b10000011	//<7>FVREN, <6>FVRRDY, <5>TSEN, <4>TSRNG, <3:2>CDAFVR, <1:0>10=2.048, 11=4.096

#define	INTCONSET	0b00000000	// <7>GIE, <6>PEIE, <5>TMR0IE, <4>INTE, <3>IOCIE, <2>TMR0IF, <1>INTF, <0>IOCIF

#define	INVLBSET	0b00000000	// <7:4>INVLB7:4, 1=ST, 0=TTL

#define	OPTIONSET	0b10001000	//<7>WPU, <6>INTEDG, <5>TMR0CS 0=Fosc/4, <4> TMR0SE <3>PSA 0=TMR0,
					//<2:0>PS 000=1:2, 011=1:16, 111=1:256

#define	OSCCONSET	0b01111010	//<7>SPLLEN, <6:3>IRCF 1011=1MHz, <2>N/A, <1:0>1x=IntOsc
					// 1011=1 MHz, 1101=4 MHz, 1111=16 MHz
//#define	OSCCONSET	0b01110000	//<7>N/A, <6:4>IRCF 111=16 MHz, <3>HFIOFR, <2>N/A, <1:0>LF/HF Osc Stable
					// 011=1 MHz, 101=4 MHz, 111=16 MHz

#define	PIE1SET		0b00000000	// <7>TMR1GIE, <6>ADIE, <5>RCIE, <4>TXIE, <3>SSP1IE, <2>CCP1IE, <1>TMR2IE, <0>TMR1IE

#define	PIE2SET		0b00000000	// <7>OSFIE, <6>C2IE, <5>C1IE, <4>EEIE, <3>BCL11IE, <2:1>N/U, <0>CCP2IE

#define	PIE3SET		0b00000000	// <7:6>N, <5>CCP4IE, <4>CCP3IE, <3>TMR6IE, <2>N, <1>TMR4IE, <0>N

#define	PIE4SET		0b00000000	// <7:2>N, <1>BCL2IE, <0>SSP2IE

#define	PIR1SET		0b00000000	// <7>TMR1GIF, <6>ADIF, <5>RCIF, <4>TXIF, <3>SSP1IF, <2>CCP1IF, <1>TMR2IF, <0>TMR1IF

#define	PIR2SET		0b00000000	// <7>OSFIF, <6>C2IF, <5>C1IF, <4>EEIF, <3>BCL1IF, <2:1>N, <0>CCP2IF

#define	PIR3SET		0b00000000	// <7:6>N, <5>CCP4IF, <4>CCP3IF, <3>TMR6IF, <2>N, <1>TMR4IF, <0>N

#define	PIR4SET		0b00000000	// <7:2>N, <1>BCL2IF, <0>SSP2IF

#define PWM1CONSET  0b00000011  // <7>PWM1RSEN (Restart), <6:0) P1DC (Delay Count)

#define T1CONSET	0b00000000      // <7:6>CS: 00-fOsc/4, 00=Inst Cycle fOsc/4
                                        // <5:4>Prescale: 00=1, <3>1=T1OSCEN, <2> T1SYNC, <1>N/A, <0>1=ON
                                        // 16 MHz clock/4: 4000000 /65536 = 61.035 Hz = 16.384 mSec

//#define	T2CONSET	0b00000010	// <6:3> Postscale 1, <0:1> Prescale 00=1 01=4 10=16 11=64
#define	T2CONSET	0b00000010	// <7>N, <6:3>T2OUTPS, <2>T2ON, <0:1> Prescale 00=1 01=4 10=16 11=64
#define	T4CONSET	0b00000010	// <7>N, <6:3>T4OUTPS, <2>T4ON, <0:1> Prescale 00=1 01=4 10=16 11=64
#define	T6CONSET	0b00000010	// <7>N, <6:3>T6OUTPS, <2>T6ON, <0:1> Prescale 00=1 01=4 10=16 11=64

#define	TRISASET	0b00011011	// Set <0:1,3:4> as input, <2,5> as output
#define	TRISBSET	0b00000000	// <7:4> TRISB7:4
#define	TRISCSET	0b00000000	// 

#define	WPUBSET     0b00000000	// <7:4> 1=PU

/*
Pin 1   VDD
Pin 2	RA5		P2A, CCP2
Pin 3	RA4		AN3, P2B
Pin 4	RA3		MCLR-, VPP
Pin 5	RC5		P1A, RX, CCP1
Pin 6	RC4		P1B, TX
Pin 7	RC3		AN7, P1C, P2A, CCP2 
Pin 8	RC6		AN8, CCP4
Pin 9	RC7		AN9
Pin 10	RB7		TX, CK, SCL2, SCK2
Pin 11	RB6		SCL1, SCK1
Pin 12	RB5     AN11, SDA2
Pin 13	RB4		AN10, SDA1
Pin 14	RC2		P1D, P2B
Pin 15	RC1     AN5, P1C
Pin 16	RC0     AN4, P1D
Pin 17	RA2		AN2, CCP3, INT
Pin 18	RA1     AN1, VREF+, ICSPCLK
Pin 19	RA0     AN0, ICSPDAT
Pin 20	VSS
 */
//#define     byte            unsigned char

#define     ConvertADC      ADCON0bits.ADGO = 1
#define     ReadADC         (int)256*(unsigned int)ADRESH + (unsigned int)ADRESL
#define     BusyADC         ADCON0bits.GO_nDONE
#define     GREEN_ON        LATAbits.LATA2 = 1;
#define     RED_ON          LATAbits.LATA2 = 0;
#define     LED_TOGGLE      LATAbits.LATA2 ^= 1;
#define     PWM_PORT        LATAbits.LATA5
#define     DisablePWM      PSTR1CONbits.STR1A = 0;
#define     EnablePWM       PSTR1CONbits.STR1A = 1;

#define     HIGH            1
#define     LOW             0

#define     CLKFREQ         16              //Clock Frequency (MHz)
#define     CLKPER          1000/CLKFREQ    //Clock period (nSec)

// 3.15V = 24.0V = 782.6 counts
// 20k + 3k / 3k = 7.67
#define     PR2_VAL         99              // 199 => 1.25 kHz, 99 => 2.5 kHz
#define     DROPDELAY       1               // mSec wait for relay drop
#define     PWM_VAL         20              // Percent PWM
#define     MAXSAMPLE       (int)2          // Number of 1 mSec samples to average
#define     ADCMIN          MAXSAMPLE*22    // (1.0V / 7.67 / 4.096) * 1024 = 22
#define     ADCPWM          MAXSAMPLE*391   // (12V * 1024 / 7.67 / 4.096) = 391.3
#define     ADCNOM          MAXSAMPLE*652   // (20V * 1024 / 7.67 / 4.096) = 652.2
#define     ADCMAX          MAXSAMPLE*913   // (28V *1024 / 7.67 / 4.096) = 913.0

/** I N C L U D E S **********************************************************/

//#include "GenericTypeDefs.h"
#include "XC.h"
#include <stdio.h>
#include <stdlib.h>
/** V A R I A B L E S ********************************************************/

//volatile BOOL buttonPressed;
//volatile BYTE buttonCount;

volatile unsigned long int      timer1=0;	// running time in 1000 uSec steps
volatile unsigned int           timer0=0;	// delay time in 1.0 mSec steps
volatile unsigned int           tm50uS=83;	// delay time in 50 uSec (1.08 degree) steps, 83 = 90 degrees
volatile unsigned char          tm0=TM0VAL;     // time in 50 uSec steps

//volatile unsigned int   timer1=0;		// running time in 1000 uSec steps
//volatile unsigned int   timer0=0;		// running time in 1000 uSec steps
//volatile unsigned char  tm0=19;                 // running time in 50 uSec steps
int                     ADCresult;		// was unsigned
int                     ADCtotal;
volatile int            AvgADC=0;
//byte                    Sample=0;

void                DelayMsec(int msec);
void                Initialize(void);
void                SetupADC(void);
void                SetupPWM(void);
void                SetupTimer0(void);
void                SetupTimer1(void);
void                SetPWM(int DC);

/** P R I V A T E  P R O T O T Y P E S ***************************************/
static void InitializeSystem(void);
void ProcessIO(void);
void YourHighPriorityISRCode();
void YourLowPriorityISRCode();
void UserInit(void);

    	//These are your actual interrupt handling routines.
	void interrupt ISRCode()
	{
	    if( INTCONbits.TMR0IF ) {       // 16 MHz, 0.25 uSec update of TMR0, PS=N/A, 200 counts = 50 uSec
        TMR0 = 256-200+13;          // =43, 50 uSec = 1.08 degrees at 60 Hz
        if( tm0 > 0)
            tm0--;
        else {                      // 100 mSec
            tm0=TM0VAL;
            if(timer0>0)
                timer0--;
            timer1++;
            }
        INTCONbits.TMR0IF = 0;
    	}
	//Check which interrupt flag caused the interrupt.
		//Service the interrupt
		//Clear the interrupt flag
		//Etc.
	}

/** DECLARATIONS ***************************************************/

    /******************************************************************************
 * Function:        void main(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        Main program entry point.
 *
 * Note:            None
 *****************************************************************************/
int main(void)
{   
    Initialize();
    SetPWM(25);
    while(1)
    {
    }//end while
}//end main

void    Initialize(void)    
    {
    OSCTUNE = 0;
    OSCCON = OSCCONSET;
    OPTION_REG = OPTIONSET;
    INTCON = INTCONSET;
    TRISA = TRISASET;
    TRISC = TRISCSET;
    LATA = 0;
    ANSELA = ANSELASET;         //Only AN4 analog
    ANSELC = ANSELCSET;
    APFCON0 = APFCON0SET;
//    IOCAN = IOCANSET;        //AN3 interrupt on change negative
//    IOCAP = IOCAPSET;        //AN3 interrupt on change positive
//    IOCAF = 0;
    INTCONbits.IOCIE = 1;       // Enable IOC
    SetupTimer0();
    SetupPWM();
}

void    SetupTimer0()
{
    INTCONbits.GIE = 0;		//Turn off Global Interrupts
    OPTION_REGbits.PSA = 1;	//0 = assigned to prescaler
    OPTION_REGbits.PS2 = 0;	//Prescale
    OPTION_REGbits.PS1 = 0;	//Prescale
    OPTION_REGbits.PS0 = 0;	//Prescale
    INTCONbits.TMR0IE = 1;
    INTCONbits.PEIE = 1;	//Turn on Peripheral Interrupts
    INTCONbits.GIE = 1;		//Turn on Global Interrupts
}

/****************************** Setup PWM - Uses TMR2, T2CON **************************/
void	SetupPWM(void)	{
    INTCONbits.GIE = 0;             //Turn off Global Interrupts
//    TRISAbits.TRISA5 = 1;           // disable output
    PR2 = PR2_VAL;                       // 16*4*15=960 uSec, 16*4*16=1024 uSec was 199
    CCP1CON = 0b10001100;           // <7:6>Half Bridge output P1A, P1B, <5:4>LSB, <3:0> All active high
    CCPR1L = 0;                     // Pulse width = 40*16=640/960 = 67%
    // Duty cycle = (CCPR2L + CCP2CON<4:5>) / 4*(PR4+1)
    // Duty cycle = 10*4 /4*15 = 40/60
    // Period = (PRx+1)*4*Tosc*TMRxPS = 960
    // 16 MHz clock PS=1=>20 kHz PWM, PS=4=>5 kHz PWM
    // 1/20 = (PR+1)*4/16000; PR+1 = 16000/(20*4) = 200; PR = 199
    PIE1bits.CCP1IE = 1;
    PIR1bits.CCP1IF = 0;
    PSTR1CON = 0;
    PSTR1CONbits.STR1B = 1;         // 1=CCP1 Output to P1B pin RA2/RA5, 0 = P1B pin is port
    PSTR1CONbits.STR1A = 1;         // 1=CCP1 Output to P1A pin RA2/RA5, 0 = P1A pin is port

    PWM1CON = PWM1CONSET;
    TRISCbits.TRISC4 = 0;           // enable output RC4
    TRISCbits.TRISC5 = 0;           // enable output RC5
    T2CON = T2CONSET;               // <6:3> Postscale 1, <0:1> Prescale 1
    T2CONbits.TMR2ON = 1;
    INTCONbits.GIE = 1;             //Turn on Global Interrupts
}

// P=(PR2+1)*4*TOSC*PS
// PW=NUM*PS*TOSC => NUM=PW*FCLK/PS
// DC=NUM/(4*(PR2+1))
// NUM=DC*(PR2+1)*4
void    SetPWM(int DC)  {           // Duty Cycle DC 0 to 100%
    unsigned long     Period;           // PWM period in nSec
//    unsigned long     PulseWidth;       // PWM pulse width in nSec
    unsigned long     Numerator;
    Period = (1000L*(PR2+1)*4*PSVAL)/CLKFREQ; // 20kHz:50000 nSec = 1000*200*4/16
//    PulseWidth = DC * Period / 100;     // 50% => 25000 = 50*50000/100
    Numerator = (DC*(PR2+1)*4)/100;  // PulseWidth*CLKFREQ/(4*PS);
//    Numerator /= 1000;
    CCP1CONbits.DC1B = Numerator & 0b00000011;
    CCPR1L = Numerator >> 2;
}

void    DelayMsec(int msec)
    {
    if(INTCONbits.TMR0IE) {
        INTCONbits.TMR0IE = 0;		//Turn off Interrupt
        tm0=TM0VAL;
        timer0=msec;
        INTCONbits.TMR0IE = 1;          //Turn on
        while(timer0>0)
            asm("NOP");
        }
    }

/** EOF main.c *************************************************/

